
REPOSITORY_QUEUE = "repositories"
PROJECT_QUEUE = "projects"

SECRET_SCANNER_TASK_NAME = "scan_repository"
LOGGING_FILE = "logging.ini"
LOG_FILE_PATH_PRJ_COLLECTOR = "project_collector.log"
LOG_FILE_PATH_REPO_COLLECTOR = "repository_collector.log"

BITBUCKET = "BITBUCKET"
AZURE_DEVOPS = "AZURE_DEVOPS"
GITHUB_PUBLIC = "GITHUB_PUBLIC"
